from django.views.generic import ListView, DetailView
from wechat_member.views import WxMemberView
from .models import Wallet, Log

class BaseView(WxMemberView):
    """
    wallet base view
    if no wallet, create it
    """
    def dispatch(self, request, *args, **kwargs):
        #super(WalletBase, self).dispatch(request, *args, **kwargs)
        try:
            result = Wallet.objects.get_or_create(
                    member_id = request.session['wx_member_id']
                    )
            self.wallet = result[0]
        except KeyError:
            """ if wx member is not exist """
            pass
        return super(BaseView, self).dispatch(request, *args, **kwargs)


class HomeView(BaseView, DetailView):
    """
    home view of wallet
    """
    model = Wallet
    template_name = 'wallet/index.html'

    def get_object(self):
        return self.wallet


class LogListView(BaseView, ListView):
    """
    wallet log list view
    """
    model = Log
    template_name = 'wallet/logs.html'

    def get_queryset(self, **kwargs):
        return Log.objects.filter(wallet=self.wallet)



from django.contrib import admin
from .models import Wallet, Log
from . import models

class LogAdmin(admin.ModelAdmin):
    list_display = ('wallet', 'description', 'money', 'created')
    list_per_page = 12
    list_filter = ('created',)
    search_fields = ('description',)

class WalletAdmin(admin.ModelAdmin):
    list_display = ('member', 'balance', 'created', 'updated')
    list_per_page = 12
    list_filter = ('created', 'updated')

admin.site.register(Wallet, WalletAdmin)
admin.site.register(Log, LogAdmin)



from .models import Wallet, Log

class WalletApi(object):
    """
    wallet api for recharge, cost, etc
    """

    def __init__(self, id):
        try:
            self.wallet = Wallet.objects.get(id=id)
        except Wallet.DoesNotExist:
            print("Wallet does not exist")
            self.wallet = None

    def balance(self):
        return self.wallet.balance

    def make_trade(self, money, description):
        balance = self.wallet.balance
        change = balance + money
        if (change) > 0:
            self.wallet.balance = change
            self.wallet.save()
        else:
            print("Balance is not enough")
            return False

        log = Log.objects.create(
                wallet = self.wallet,
                money = money,
                description = description,
                )
        return True



from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _
from django.apps import AppConfig

class WalletConfig(AppConfig):
    name = 'wallet'
    verbose_name = _('wallet')



from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _
from django.db import models
from wechat_member.models import Member

class Wallet(models.Model):
    """
    Wallet based on wechat member
    """
    member = models.OneToOneField(Member, related_name='wallet',\
            verbose_name=_('member'))
    balance = models.DecimalField(_('balance'), max_digits=8, \
            decimal_places=2, default=0)
    created = models.DateTimeField(_('created'), auto_now_add=True)
    updated = models.DateTimeField(_('updated'), auto_now=True)

    def __unicode__(self):
        return self.member.name

    def __str__(self):
        return self.member.name

    class Meta(object):
        verbose_name = _('wallet')
        verbose_name_plural = _('wallet')


class Log(models.Model):
    """
    Wallet transaction log
    """
    wallet = models.ForeignKey(Wallet, related_name='logs',\
            verbose_name=_('wallet'))
    money = models.DecimalField(_('money'), max_digits=8, decimal_places=2)
    description = models.CharField(_('description'), max_length=200)
    created = models.DateTimeField(_('created'), auto_now_add=True)

    def __unicode__(self):
        return self.description

    def __str__(self):
        return self.description

    class Meta(object):
        verbose_name = _('log')
        verbose_name_plural = _('log')
        ordering = ['-created']
